{
  MACAL (Mass Calculator)
  Copyright (C) 2007, Innsbruck Medical University

  Author: Stefan Ascher <stefan.ascher@i-med.ac.at>

  $Id: uMasses.pas,v 1.4 2007/04/05 07:51:00 q002sa Exp $
}

unit uMasses;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

interface

uses
  System.SysUtils, Classes;
  
type
  TMassType = (mtAvg, mtMono);
  TMassList = class;
  TMass = class
  private
    fName: string;
    fMassAvg: double;
    fMassMono: double;
    function GetMassType: TMassType;
  protected
    fOwner: TMassList;
    fChar: Char;
    fComposition: string;
    function GetMass: double; virtual;
    procedure SetMass(Value: double);
  public
    constructor Create(AOwner: TMassList);
    property NameChar: Char read fChar write fChar;
    property Name: string read fName write fName;
    property Composition: string read fComposition write fComposition;
    property Mass: double read GetMass;
    property MassMono: double read fMassMono write fMassMono;
    property MassAvg: double read fMassAvg write fMassAvg;
    property MassType: TMassType read GetMassType;
  end;

  TMassList = class(TList)
  private
  protected
    fFilename: string;
    fMassType: TMassType;
    function Get(Index: Integer): TMass;
    procedure Put(Index: Integer; Item: TMass);
  public
    procedure DeleteAll;
    property Items[Index: Integer]: TMass read Get write Put; default;
    function Find(const AChar: Char): TMass;
    procedure LoadFromFile(const AFilename: string); virtual;
    property MassType: TMassType read fMassType write fMassType;
    property Filename: string read fFilename;
  end;

const
  MASS_FILE = 'masses.ini';

function StrToMass(const Str: string): double;

implementation

function StrToMass(const Str: string): double;
var
  s: string;
begin
  if Str = '' then begin
    Result := 0.0;
    Exit;
  end;
  s := StringReplace(Str, '.', FormatSettings.DecimalSeparator, [rfReplaceall]);
  s := StringReplace(s, ',', FormatSettings.DecimalSeparator, [rfReplaceall]);
  Result := StrToFloat(s);
end;

{ TMassList }

procedure TMassList.DeleteAll;
var
  i: integer;
begin
  for i := 0 to Count - 1 do
    Items[i].Free;
  Clear;
end;

function TMassList.Find(const AChar: Char): TMass;
var
  i: integer;
begin
  Result := nil;
  for i := 0 to Count - 1 do begin
    if Items[i].NameChar = AChar then begin
      Result := Items[i];
      Break;
    end;
  end;
end;

function TMassList.Get(Index: Integer): TMass;
begin
  Result := TMass(inherited Get(Index));
end;

procedure TMassList.Put(Index: Integer; Item: TMass);
begin
  inherited Put(Index, Item);
end;

procedure TMassList.LoadFromFile(const AFilename: string);
var
  sl: TStringList;
  i, p1: integer;
  str, str2: string;
  aa: TMass;
begin
  fFilename := AFilename;
  Clear;
  if not FileExists(AFilename) then Exit;
  sl := TStringList.Create;
  try
    sl.LoadFromFile(AFilename);
    for i := 0 to sl.Count - 1 do begin
      str := Trim(sl[i]);
      if (str <> '') and (str[1] <> ';') then begin
        aa := TMass.Create(Self);
        Add(aa);
        aa.NameChar := str[1];
        System.Delete(str, 1, 2);

        p1 := Pos(';', str);
        if (p1 = 0) then
          Continue;
        str2 := Copy(str, 1, p1-1);
        aa.Name := str2;

        System.Delete(str, 1, p1);
        p1 := Pos(';', str);
        if (p1 = 0) then
          Continue;
        str2 := Copy(str, 1, p1-1);
        aa.MassMono := StrToMass(str2);

        System.Delete(str, 1, p1);
        if (str = '') then
          Continue;
        aa.MassAvg := StrToMass(str);
      end;
    end;
  finally
    sl.Free;
  end;
end;

{ TMass }

constructor TMass.Create;
begin
  inherited Create;
  fOwner := AOwner;
end;

function TMass.GetMass: double;
begin
  if fOwner.MassType = mtMono then
    Result := fMassMono
  else
    Result := fMassAvg;
end;

function TMass.GetMassType: TMassType;
begin
  Result := fOwner.MassType;
end;

procedure TMass.SetMass(Value: double);
begin
  if fOwner.MassType = mtMono then
    fMassMono := Value
  else
    fMassAvg := Value;
end;

end.
