{
  MACAL (Mass Calculator)
  Copyright (C) 2007, Innsbruck Medical University

  Author: Stefan Ascher <stefan.ascher@i-med.ac.at>

  $Id: uMods.pas,v 1.5 2007/04/05 07:51:00 q002sa Exp $
}

unit uMods;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

interface

uses
  SysUtils, Classes, uMasses;

type
  TTerminus = (tC, tN);
  TTermini = set of TTerminus;

  TMod = class(TMass)
  private
    fResidues: string;
    fTermini: TTermini;
    fSelected: boolean;
    function GetFixed: boolean;
  protected
    function GetMass: double; override;
  public
    constructor Create(AOwner: TMassList);
    property Residues: string read fResidues write fResidues;
    property Fixed: boolean read GetFixed;
    property Termini: TTermini read fTermini write fTermini;
    property Selected: boolean read fSelected write fSelected;
  end;

  TModList = class(TMassList)
  public
    procedure LoadFromFile(const AFilename: string); override;
    function GetFromAA(const AA: Char): TMod;
  end;

const
  MODS_FILE = 'mods.ini';

implementation

{ TModList }

procedure TModList.LoadFromFile(const AFilename: string);
var
  sl: TStringList;
  i, p1: integer;
  str, str2: string;
  aa: TMod;
  t: TTermini;
begin
  Clear;
  fFilename := AFilename;
  if not FileExists(AFilename) then Exit;
  sl := TStringList.Create;
  try
    sl.LoadFromFile(AFilename);
    for i := 0 to sl.Count - 1 do begin
      str := Trim(sl[i]);
      if (str <> '') and (str[1] <> ';') then begin
        aa := TMod.Create(Self);
        Add(aa);
        aa.NameChar := str[1];
        System.Delete(str, 1, 2);

        p1 := Pos(';', str);
        if (p1 = 0) then
          Continue;
        str2 := Copy(str, 1, p1-1);
        aa.Name := str2;

        System.Delete(str, 1, p1);
        p1 := Pos(';', str);
        if (p1 = 0) then
          Continue;
        str2 := Copy(str, 1, p1-1);
        aa.Composition := str2;

        System.Delete(str, 1, p1);
        p1 := Pos(';', str);
        if (p1 = 0) then
          Continue;
        str2 := Copy(str, 1, p1-1);
        aa.Residues := str2;

        System.Delete(str, 1, p1);
        p1 := Pos(';', str);
        if (p1 = 0) then
          Continue;
        str2 := Copy(str, 1, p1-1);
        if (str2 <> '') then
          aa.MassMono := StrToMass(str2)
        else
          aa.MassMono := 0.0;

        System.Delete(str, 1, p1);
        p1 := Pos(';', str);
        if (p1 = 0) then
          Continue;
        str2 := Copy(str, 1, p1-1);
        if (str2 <> '') then
          aa.MassAvg := StrToMass(str2)
        else
          aa.MassAvg := 0.0;

        System.Delete(str, 1, p1);
        p1 := Pos(';', str);
        if (p1 = 0) then
          Continue;
        str2 := Copy(str, 1, p1-1);
        t := [];
        if Pos('N', str2) <> 0 then
          Include(t, tN);
        if Pos('C', str2) <> 0 then
          Include(t, tC);
        aa.Termini := t;
      end;
    end;
  finally
    sl.Free;
  end;
end;

function TModList.GetFromAA(const AA: Char): TMod;
var
  i: integer;
  m: TMod;
begin
  Result := nil;
  for i := 0 to Count - 1 do begin
    m := TMod(Items[i]);
    if (Pos(AA, m.Residues) <> 0) and (m.Selected) then begin
      Result := m;
      Break;
    end;
  end;
end;

{ TMod }

constructor TMod.Create(AOwner: TMassList);
begin
  inherited;
  fSelected := false;
end;

function TMod.GetFixed: boolean;
begin
  Result := (NameChar = '-');
end;

function TMod.GetMass: double;
begin
  if (fSelected) then
    Result := inherited GetMass
  else
    Result := 0.0;
end;

end.
